var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
import { Component, Input, Output, ElementRef, ViewChild, Renderer2, EventEmitter } from '@angular/core';
import { QRCode } from '../lib/qr-decoder/qrcode';
var QrCodeScanComponent = /** @class */ (function () {
    function QrCodeScanComponent(renderer) {
        this.renderer = renderer;
        this.canvasWidth = 640;
        this.canvasHeight = 480;
        this.debug = false;
        this.stopAfterScan = true;
        this.updateTime = 500;
        this.onCapture = new EventEmitter();
        this.hideCanvas = true;
    }
    Object.defineProperty(QrCodeScanComponent.prototype, "isCanvasSupported", {
        get: function () {
            var tempCanvas = this.renderer.createElement('canvas');
            return !!(tempCanvas && tempCanvas.getContext && tempCanvas.getContext('2d'));
        },
        set: function (value) {
            this.isCanvasSupported = value;
        },
        enumerable: true,
        configurable: true
    });
    QrCodeScanComponent.prototype.ngOnInit = function () {
    };
    QrCodeScanComponent.prototype.ngAfterViewInit = function () {
        var _this = this;
        if (this.isCanvasSupported) {
            if (this.debug)
                console.debug('The canvas is supprted on your browser');
            this.gCtx = this.canvasRef.nativeElement.getContext('2d');
            this.gCtx.clearRect(0, 0, this.canvasWidth, this.canvasHeight);
            this.qrcode = new QRCode();
            if (this.debug)
                this.qrcode.debug = true;
            this.qrcode.myCallback = function (decoded) { _this.QrCodeCallback(decoded); };
            this.initCameraComponents();
        }
    };
    QrCodeScanComponent.prototype.ngOnDestroy = function () {
    };
    QrCodeScanComponent.prototype.initCameraComponents = function () {
        var _this = this;
        var browser = navigator, constraints = {
            audio: false,
            video: true
        };
        if (this.stopTimeout) {
            this.stopScanning();
        }
        if (!this.videoElement) {
            this.videoElement = this.renderer.createElement('video');
            this.videoElement.setAttribute('autoplay', 'true');
            this.videoElement.setAttribute('muted', 'true');
            this.renderer.appendChild(this.videoRef.nativeElement, this.videoElement);
        }
        browser.getUserMedia = (browser.getUserMedia ||
            browser.webkitGetUserMedia ||
            browser.mozGetUserMedia ||
            browser.msGetUserMedia);
        browser.mediaDevices.getUserMedia(constraints)
            .then(function (stream) {
            _this.setStream(stream);
        })
            .catch(function (err) {
            console.error(err);
            _this.isCanvasSupported = false;
        });
    };
    QrCodeScanComponent.prototype.QrCodeCallback = function (decodedData) {
        var _this = this;
        if (this.stopAfterScan) {
            this.onCapture.next(decodedData);
            this.stopScanning();
        }
        else {
            this.onCapture.next(decodedData);
            this.stopTimeout = setTimeout(function () { return _this.captureCanvas(); }, this.updateTime);
        }
    };
    QrCodeScanComponent.prototype.setStream = function (stream) {
        var _this = this;
        this.hideCanvas = true;
        this.gCtx.clearRect(0, 0, this.canvasWidth, this.canvasHeight);
        this.stream = stream;
        if (this.videoElement)
            this.videoElement.srcObject = stream;
        this.stopTimeout = setTimeout(function () { return _this.captureCanvas(); }, this.updateTime);
    };
    QrCodeScanComponent.prototype.captureCanvas = function () {
        var _this = this;
        try {
            this.gCtx.drawImage(this.videoElement, this.canvasWidth, this.canvasHeight);
            this.qrcode.decode(this.canvasRef.nativeElement);
        }
        catch (e) {
            this.debug && console.error(e);
            this.stopTimeout = setTimeout(function () { return _this.captureCanvas(); }, this.updateTime);
        }
    };
    QrCodeScanComponent.prototype.stopScanning = function () {
        if (this.stopTimeout) {
            clearTimeout(this.stopTimeout);
            this.stopTimeout = 0;
        }
        this.hideCanvas = false;
        if (this.stream && this.stream.getTracks().length) {
            this.stream.getTracks().forEach(function (device) {
                device && device.enabled && device.stop();
            });
            this.stream = null;
        }
    };
    __decorate([
        Input('canvasWidth'),
        __metadata("design:type", Number)
    ], QrCodeScanComponent.prototype, "canvasWidth", void 0);
    __decorate([
        Input('canvasHeight'),
        __metadata("design:type", Number)
    ], QrCodeScanComponent.prototype, "canvasHeight", void 0);
    __decorate([
        Input('debug'),
        __metadata("design:type", Boolean)
    ], QrCodeScanComponent.prototype, "debug", void 0);
    __decorate([
        Input('stopAfterScan'),
        __metadata("design:type", Boolean)
    ], QrCodeScanComponent.prototype, "stopAfterScan", void 0);
    __decorate([
        Input('updateTime'),
        __metadata("design:type", Number)
    ], QrCodeScanComponent.prototype, "updateTime", void 0);
    __decorate([
        Output('onCapture'),
        __metadata("design:type", Object)
    ], QrCodeScanComponent.prototype, "onCapture", void 0);
    __decorate([
        ViewChild('qrCodeCanvas'),
        __metadata("design:type", ElementRef)
    ], QrCodeScanComponent.prototype, "canvasRef", void 0);
    __decorate([
        ViewChild('videoWrapper'),
        __metadata("design:type", ElementRef)
    ], QrCodeScanComponent.prototype, "videoRef", void 0);
    QrCodeScanComponent = __decorate([
        Component({
            selector: 'app-qrcode-scan',
            template: "\n        <ng-container [ngSwitch]=\"isCanvasSupported\">\n            <ng-container *ngSwitchDefault>\n                <canvas #qrCodeCanvas [hidden]=\"hideCanvas\" [width]=\"canvasWidth\" [height]=\"canvasHeight\"></canvas>\n                <div #videoWrapper [style.width]=\"canvasWidth\" [style.height]=\"canvasHeight\"></div>\n            </ng-container>\n            <ng-container *ngSwitchCase=\"false\">\n                <p>You are using an <strong>outdated</strong> browser</p>\n            </ng-container>\n        </ng-container>\n    ",
            styles: [
                ':host video {height: auto, width: 100%}',
                ':host {}'
            ]
        }),
        __metadata("design:paramtypes", [Renderer2])
    ], QrCodeScanComponent);
    return QrCodeScanComponent;
}());
export { QrCodeScanComponent };
//# sourceMappingURL=qrCodeScan.component.js.map